@@ -1928,8 +1928,11 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 			switch (scsi_error(d, s.sbp, s.sb_len_wr)) {
 			case RETRY:
 				done = 0;
-				if (burn_sg_log_scsi & 3)
+				if (burn_sg_log_scsi & 3) {
+					scsi_log_err(c, fp, s.sbp, s.duration,
+								c->error != 0);
 					scsi_log_cmd(c,fp,0);
+				}
 				break;
 			case FAIL:
 				done = 1;
