@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
+ * Copyright (c) 2010 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -17,9 +18,10 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 
 /**
- * This table should be written with accuracy values at offset
+ * This table should be written with the actual values at offset
  * 8 of boot image, when used ISOLINUX boot loader
  */
 struct boot_info_table {
@@ -55,6 +57,19 @@ struct hard_disc_mbr {
     uint8_t sign2;
 };
 
+/* API */
+int el_torito_set_boot_platform_id(ElToritoBootImage *bootimg, uint8_t id)
+{
+    bootimg->platform_id = id;
+    return 1;
+}
+
+/* API */
+int el_torito_get_boot_platform_id(ElToritoBootImage *bootimg)
+{
+    return bootimg->platform_id;
+}
+
 /**
  * Sets the load segment for the initial boot image. This is only for
  * no emulation boot images, and is a NOP for other image types.
@@ -66,6 +81,14 @@ void el_torito_set_load_seg(ElToritoBootImage *bootimg, short segment)
     bootimg->load_seg = segment;
 }
 
+/* API */
+int el_torito_get_load_seg(ElToritoBootImage *bootimg)
+{
+   if (bootimg->load_seg < 0)
+       return 0xffff - bootimg->load_seg;
+   return bootimg->load_seg;
+}
+ 
 /**
  * Sets the number of sectors (512b) to be load at load segment during
  * the initial boot procedure. This is only for no emulation boot images,
@@ -78,6 +101,14 @@ void el_torito_set_load_size(ElToritoBootImage *bootimg, short sectors)
     bootimg->load_size = sectors;
 }
 
+/* API */
+int el_torito_get_load_size(ElToritoBootImage *bootimg)
+{
+   if (bootimg->load_size < 0)
+       return 0xffff - bootimg->load_size;
+   return bootimg->load_size;
+}
+
 /**
  * Marks the specified boot image as not bootable
  */
@@ -86,8 +117,50 @@ void el_torito_set_no_bootable(ElToritoBootImage *bootimg)
     bootimg->bootable = 0;
 }
 
+/* API */
+int el_torito_get_bootable(ElToritoBootImage *bootimg)
+{   
+    return !!bootimg->bootable;
+}
+
+/* API */
+int el_torito_set_id_string(ElToritoBootImage *bootimg, uint8_t id_string[28])
+{
+    memcpy(bootimg->id_string, id_string, 28);
+    return 1;
+}
+
+/* API */
+int el_torito_get_id_string(ElToritoBootImage *bootimg, uint8_t id_string[28])
+{
+    
+    memcpy(id_string, bootimg->id_string, 28);
+    return 1;
+}
+
+/* API */
+int el_torito_set_selection_crit(ElToritoBootImage *bootimg, uint8_t crit[20])
+{
+    memcpy(bootimg->selection_crit, crit, 20);
+    return 1;
+}
+
+/* API */
+int el_torito_get_selection_crit(ElToritoBootImage *bootimg, uint8_t crit[20])
+{
+    
+    memcpy(crit, bootimg->selection_crit, 20);
+    return 1;
+}
+
+/* API */
+int el_torito_seems_boot_info_table(ElToritoBootImage *bootimg, int flag)
+{
+    return bootimg->seems_boot_info_table;
+}
+
 /**
- * Specifies that this image needs to be patched. This involves the writting
+ * Specifies that this image needs to be patched. This involves the writing
  * of a 56 bytes boot information table at offset 8 of the boot image file.
  * The original boot image file won't be modified.
  * This is needed for isolinux boot images.
@@ -106,7 +179,7 @@ void el_torito_patch_isolinux_image(ElToritoBootImage *bootimg)
  *        bitmask style flag. The following values are defined:
  *
  *        bit 0 -> 1 to path the image, 0 to not
- *                 Patching the image involves the writting of a 56 bytes
+ *                 Patching the image involves the writing of a 56 bytes
  *                 boot information table at offset 8 of the boot image file.
  *                 The original boot image file won't be modified. This is needed
  *                 to allow isolinux images to be bootable.
@@ -124,31 +197,36 @@ int el_torito_set_isolinux_options(ElToritoBootImage *bootimg, int options, int
     return ISO_SUCCESS;
 }
 
-/* TODO getter for boot image properties should be exposed
- * useful when reading discs */
-int el_torito_get_boot_media_type(const ElToritoBootImage *bootimg)
+/* API */
+int el_torito_get_isolinux_options(ElToritoBootImage *bootimg, int flag)
+{
+    return bootimg->isolinux_options & 0x03;
+}
+
+/* API */
+int el_torito_get_boot_media_type(ElToritoBootImage *bootimg,
+                                  enum eltorito_boot_media_type *media_type)
 {
     if (bootimg) {
         switch (bootimg->type) {
         case 1:
         case 2:
         case 3:
-            return ELTORITO_FLOPPY_EMUL;
-            break;
+            *media_type = ELTORITO_FLOPPY_EMUL;
+            return 1;
         case 4:
-            return ELTORITO_HARD_DISC_EMUL;
-            break;
+            *media_type = ELTORITO_HARD_DISC_EMUL;
+            return 1;
         case 0:
-            return ELTORITO_NO_EMUL;
-            break;
+            *media_type = ELTORITO_NO_EMUL;
+            return 1;
         default:
             /* should never happen */
             return ISO_ASSERT_FAILURE;
             break;
         }
-    } else {
-        return ISO_WRONG_ARG_VALUE;
     }
+    return ISO_WRONG_ARG_VALUE;
 }
 
 static
@@ -333,12 +411,15 @@ int create_image(IsoImage *image, const char *image_path,
     boot->image = (IsoFile*)imgfile;
     iso_node_ref(imgfile); /* get our ref */
     boot->bootable = 1;
+    boot->seems_boot_info_table = 0;
     boot->isolinux_options = 0;
     boot->type = boot_media_type;
     boot->partition_type = partition_type;
     boot->load_seg = 0;
     boot->load_size = load_sectors;
-
+    boot->platform_id = 0; /* 80x86 */
+    memset(boot->id_string, 0, sizeof(boot->id_string));
+    memset(boot->selection_crit, 0, sizeof(boot->selection_crit));
     if (bootimg) {
         *bootimg = boot;
     }
@@ -351,7 +432,7 @@ int iso_image_set_boot_image(IsoImage *image, const char *image_path,
                              const char *catalog_path,
                              ElToritoBootImage **boot)
 {
-    int ret;
+    int ret, i;
     struct el_torito_boot_catalog *catalog;
     ElToritoBootImage *boot_image= NULL;
     IsoBoot *cat_node= NULL;
@@ -415,8 +496,12 @@ int iso_image_set_boot_image(IsoImage *image, const char *image_path,
         ret = ISO_OUT_OF_MEM;
         goto boot_image_cleanup;
     }
-    catalog->image = boot_image;
+    catalog->num_bootimages = 1;
+    catalog->bootimages[0] = boot_image;
+    for (i = 1; i < Libisofs_max_boot_imageS; i++)
+        catalog->bootimages[i] = NULL;
     catalog->node = cat_node;
+    catalog->sort_weight = 1000;                            /* slightly high */
     iso_node_ref((IsoNode*)cat_node);
     image->bootcat = catalog;
 
@@ -439,7 +524,7 @@ boot_image_cleanup:;
 }
 
 /**
- * Get El-Torito boot image of an ISO image, if any.
+ * Get the boot catalog and the El-Torito default boot image of an ISO image.
  *
  * This can be useful, for example, to check if a volume read from a previous
  * session or an existing image is bootable. It can also be useful to get
@@ -483,10 +568,10 @@ int iso_image_get_boot_image(IsoImage *image, ElToritoBootImage **boot,
 
     /* ok, image is bootable */
     if (boot) {
-        *boot = image->bootcat->image;
+        *boot = image->bootcat->bootimages[0];
     }
     if (imgnode) {
-        *imgnode = image->bootcat->image->image;
+        *imgnode = image->bootcat->bootimages[0]->image;
     }
     if (catnode) {
         *catnode = image->bootcat->node;
@@ -494,6 +579,40 @@ int iso_image_get_boot_image(IsoImage *image, ElToritoBootImage **boot,
     return ISO_SUCCESS;
 }
 
+int iso_image_get_all_boot_imgs(IsoImage *image, int *num_boots,
+               ElToritoBootImage ***boots, IsoFile ***bootnodes, int flag)
+{
+    int i;
+    struct el_torito_boot_catalog *cat;
+
+    if (image == NULL)
+        return ISO_NULL_POINTER;
+    if (image->bootcat == NULL)
+        return 0;
+    cat = image->bootcat;
+    *num_boots = cat->num_bootimages;
+    *boots = NULL;
+    *bootnodes = NULL;
+    if (*num_boots <= 0)
+        return 0;
+    *boots = calloc(*num_boots, sizeof(ElToritoBootImage *));
+    *bootnodes = calloc(*num_boots, sizeof(IsoFile *));
+    if(*boots == NULL || *bootnodes == NULL) {
+        if (*boots != NULL)
+            free(*boots);
+        if (*bootnodes != NULL)
+            free(*bootnodes);
+        *boots = NULL;
+        *bootnodes = NULL;
+        return ISO_OUT_OF_MEM;
+    }
+    for (i = 0; i < *num_boots; i++) {
+        (*boots)[i] = cat->bootimages[i];
+        (*bootnodes)[i] = image->bootcat->bootimages[i]->image;
+    }
+    return 1;
+}
+
 /**
  * Removes the El-Torito bootable image.
  *
@@ -508,28 +627,68 @@ void iso_image_remove_boot_image(IsoImage *image)
         return;
 
     /*
-     * remove catalog node from its parent
-     * (the reference will be disposed next)
+     * remove catalog node from its parent and dispose it
+     * (another reference is with the catalog)
      */
-    iso_node_take((IsoNode*)image->bootcat->node);
+    if (iso_node_get_parent((IsoNode*) image->bootcat->node) != NULL) {
+        iso_node_take((IsoNode*) image->bootcat->node);
+        iso_node_unref((IsoNode*) image->bootcat->node);
+    }
 
     /* free boot catalog and image, including references to nodes */
     el_torito_boot_catalog_free(image->bootcat);
     image->bootcat = NULL;
 }
 
+/* API */
+int iso_image_add_boot_image(IsoImage *image, const char *image_path,
+                             enum eltorito_boot_media_type type, int flag,
+                             ElToritoBootImage **boot)
+{
+    int ret;
+    struct el_torito_boot_catalog *catalog = image->bootcat;
+    ElToritoBootImage *boot_img;
+
+    if (catalog->num_bootimages >= Libisofs_max_boot_imageS)
+        return ISO_BOOT_IMAGE_OVERFLOW;
+    ret = create_image(image, image_path, type, &boot_img);
+    if (ret < 0) 
+        return ret;
+    catalog->bootimages[catalog->num_bootimages] = boot_img;
+    catalog->num_bootimages++;
+    if (boot != NULL)
+        *boot = boot_img;
+    return 1;
+}
+
+/* API */
+int iso_image_set_boot_catalog_weight(IsoImage *image, int sort_weight)
+{
+    if (image->bootcat == NULL)
+        return 0;
+    image->bootcat->sort_weight = sort_weight;
+    return 1;
+}
+
 void el_torito_boot_catalog_free(struct el_torito_boot_catalog *cat)
 {
     struct el_torito_boot_image *image;
+    int i;
 
     if (cat == NULL) {
         return;
     }
 
-    image = cat->image;
-    iso_node_unref((IsoNode*)image->image);
-    free(image);
-    iso_node_unref((IsoNode*)cat->node);
+    for (i = 0; i < Libisofs_max_boot_imageS; i++) {
+        image = cat->bootimages[i];
+        if (image == NULL)
+    continue;
+        if ((IsoNode*)image->image != NULL)
+            iso_node_unref((IsoNode*)image->image);
+        free(image);
+    }
+    if ((IsoNode*)cat->node != NULL)
+        iso_node_unref((IsoNode*)cat->node);
     free(cat);
 }
 
@@ -540,11 +699,12 @@ struct catalog_stream
 {
     Ecma119Image *target;
     uint8_t buffer[BLOCK_SIZE];
-    int offset; /* -1 if stream is not openned */
+    int offset; /* -1 if stream is not opened */
 };
 
 static void
-write_validation_entry(uint8_t *buf)
+write_validation_entry(uint8_t *buf, uint8_t platform_id,
+                       uint8_t id_string[24])
 {
     size_t i;
     int checksum;
@@ -552,10 +712,10 @@ write_validation_entry(uint8_t *buf)
     struct el_torito_validation_entry *ve =
         (struct el_torito_validation_entry*)buf;
     ve->header_id[0] = 1;
-    ve->platform_id[0] = 0; /* 0: 80x86, 1: PowerPC, 2: Mac */
+    ve->platform_id[0] = platform_id;
+    memcpy(ve->id_string, id_string, sizeof(ve->id_string));
     ve->key_byte1[0] = 0x55;
     ve->key_byte2[0] = 0xAA;
-
     /* calculate the checksum, to ensure sum of all words is 0 */
     checksum = 0;
     for (i = 0; i < sizeof(struct el_torito_validation_entry); i += 2) {
@@ -564,35 +724,64 @@ write_validation_entry(uint8_t *buf)
     iso_lsb(ve->checksum, checksum, 2);
 }
 
+static void
+write_section_header(uint8_t *buf, Ecma119Image *t, int idx, int num_entries)
+{
+    int pi;
+    char *id_string;
+
+    struct el_torito_section_header *e =
+        (struct el_torito_section_header *) buf;
+
+    /* 0x90 = more section headers follow , 0x91 = final section */
+    e->header_indicator[0] = 0x90 + (idx == t->catalog->num_bootimages - 1);
+    pi= e->platform_id[0] = t->catalog->bootimages[idx]->platform_id;
+    e->num_entries[0] = num_entries & 0xff;
+    e->num_entries[1] = (num_entries >> 8) & 0xff;;
+    id_string = (char *) e->id_string;
+    memcpy(id_string,  t->catalog->bootimages[idx]->id_string,
+           sizeof(e->id_string));
+}
+
 /**
  * Write one section entry.
- * Currently this is used only for default image (the only supported just now)
+ * Usable for the Default Entry
+ * and for Section Entries with Selection criteria type == 0
  */
 static void
-write_section_entry(uint8_t *buf, Ecma119Image *t)
+write_section_entry(uint8_t *buf, Ecma119Image *t, int idx)
 {
     struct el_torito_boot_image *img;
     struct el_torito_section_entry *se =
         (struct el_torito_section_entry*)buf;
 
-    img = t->catalog->image;
+    img = t->catalog->bootimages[idx];
 
     se->boot_indicator[0] = img->bootable ? 0x88 : 0x00;
     se->boot_media_type[0] = img->type;
     iso_lsb(se->load_seg, img->load_seg, 2);
     se->system_type[0] = img->partition_type;
     iso_lsb(se->sec_count, img->load_size, 2);
-    iso_lsb(se->block, t->bootimg->sections[0].block, 4);
+    iso_lsb(se->block, t->bootsrc[idx]->sections[0].block, 4);
+    se->selec_criteria[0] = img->selection_crit[0];
+    memcpy(se->vendor_sc, img->selection_crit + 1, 19);
 }
 
 static
 int catalog_open(IsoStream *stream)
 {
+    int i, j, k, num_entries;
     struct catalog_stream *data;
+    uint8_t *wpt;
+    struct el_torito_boot_catalog *cat;
+    struct el_torito_boot_image **boots;
+
     if (stream == NULL) {
         return ISO_NULL_POINTER;
     }
     data = stream->data;
+    cat = data->target->catalog;
+    boots = cat->bootimages;
 
     if (data->offset != -1) {
         return ISO_FILE_ALREADY_OPENED;
@@ -601,11 +790,36 @@ int catalog_open(IsoStream *stream)
     memset(data->buffer, 0, BLOCK_SIZE);
 
     /* fill the buffer with the catalog contents */
-    write_validation_entry(data->buffer);
+    write_validation_entry(data->buffer,
+                           boots[0]->platform_id, boots[0]->id_string);
 
-    /* write default entry */
-    write_section_entry(data->buffer + 32, data->target);
+    /* write default entry = first boot image */
+    write_section_entry(data->buffer + 32, data->target, 0);
 
+    /* IMPORTANT: The maximum number of boot images must fit into BLOCK_SIZE */
+    wpt = data->buffer + 64;
+    for (i = 1; i < cat->num_bootimages; ) {
+        /* Look ahead and put images of same platform_id and id_string
+           into the same section */
+        for (j = i + 1; j < cat->num_bootimages; j++) {
+             if (boots[i]->platform_id != boots[j]->platform_id)
+        break;
+             for (k = 0; k < sizeof(boots[i]->id_string); k++)
+                 if (boots[i]->id_string[k] != boots[j]->id_string[k])
+             break;
+             if (k < sizeof(boots[i]->id_string))
+        break;
+        }
+        num_entries = j - i;
+
+        write_section_header(wpt, data->target, i, num_entries);
+        wpt += 32;
+        for (j = 0; j < num_entries; j++) {
+            write_section_entry(wpt,  data->target, i);
+            wpt += 32;
+            i++;
+        }
+    }
     data->offset = 0;
     return ISO_SUCCESS;
 }
@@ -759,7 +973,7 @@ int el_torito_catalog_file_src_create(Ecma119Image *target, IsoFileSrc **src)
     file->checksum_index = 0;
     file->nsections = 1;
     file->sections = calloc(1, sizeof(struct iso_file_section));
-    file->sort_weight = 1000; /* slightly high */
+    file->sort_weight = target->catalog->sort_weight;
     file->stream = stream;
 
     ret = iso_file_src_add(target, file, src);
@@ -775,27 +989,26 @@ int el_torito_catalog_file_src_create(Ecma119Image *target, IsoFileSrc **src)
 /******************* EL-TORITO WRITER *******************************/
 
 /**
- * Patch an isolinux boot image.
+ * Insert boot info table content into buf.
  *
  * @return
  *      1 on success, 0 error (but continue), < 0 error
  */
-static
-int patch_boot_image(uint8_t *buf, Ecma119Image *t, size_t imgsize)
+int make_boot_info_table(uint8_t *buf, uint32_t pvd_lba,
+                         uint32_t boot_lba, uint32_t imgsize)
 {
     struct boot_info_table *info;
     uint32_t checksum;
-    size_t offset;
+    uint32_t offset;
 
-    if (imgsize < 64) {
-        return iso_msg_submit(t->image->id, ISO_ISOLINUX_CANT_PATCH, 0,
-            "Isolinux image too small. We won't patch it.");
-    }
+    info = (struct boot_info_table *) (buf + 8);
+    if (imgsize < 64)
+        return ISO_ISOLINUX_CANT_PATCH;
 
     /* compute checksum, as the the sum of all 32 bit words in boot image
      * from offset 64 */
     checksum = 0;
-    offset = (size_t) 64;
+    offset = 64;
 
     while (offset <= imgsize - 4) {
         checksum += iso_read_lsb(buf + offset, 4);
@@ -810,16 +1023,36 @@ int patch_boot_image(uint8_t *buf, Ecma119Image *t, size_t imgsize)
         checksum += iso_read_lsb(buf + offset, imgsize - offset);
     }
 
-    /* patch boot info table */
-    info = (struct boot_info_table*)(buf + 8);
     /*memset(info, 0, sizeof(struct boot_info_table));*/
-    iso_lsb(info->bi_pvd, t->ms_block + 16, 4);
-    iso_lsb(info->bi_file, t->bootimg->sections[0].block, 4);
+    iso_lsb(info->bi_pvd, pvd_lba, 4);
+    iso_lsb(info->bi_file, boot_lba, 4);
     iso_lsb(info->bi_length, imgsize, 4);
     iso_lsb(info->bi_csum, checksum, 4);
+    memset(buf + 24, 0, 40);
     return ISO_SUCCESS;
 }
 
+/**
+ * Patch an isolinux boot image.
+ *
+ * @return
+ *      1 on success, 0 error (but continue), < 0 error
+ */
+static
+int patch_boot_image(uint8_t *buf, Ecma119Image *t, size_t imgsize, int idx)
+{
+    int ret;
+
+    if (imgsize < 64) {
+        return iso_msg_submit(t->image->id, ISO_ISOLINUX_CANT_PATCH, 0,
+            "Isolinux image too small. We won't patch it.");
+    }
+    ret = make_boot_info_table(buf, t->ms_block + (uint32_t) 16,
+                               t->bootsrc[idx]->sections[0].block,
+                               (uint32_t) imgsize);
+    return ret;
+}
+
 static
 int eltorito_writer_compute_data_blocks(IsoImageWriter *writer)
 {
@@ -828,7 +1061,11 @@ int eltorito_writer_compute_data_blocks(IsoImageWriter *writer)
      * this is a good place to do so.
      */
     Ecma119Image *t;
-    int ret;
+    int ret, idx;
+    size_t size;
+    uint8_t *buf;
+    IsoStream *new = NULL;
+    IsoStream *original = NULL;
 
     if (writer == NULL) {
         return ISO_NULL_POINTER;
@@ -836,13 +1073,16 @@ int eltorito_writer_compute_data_blocks(IsoImageWriter *writer)
 
     t = writer->target;
 
-    if (t->catalog->image->isolinux_options & 0x01) {
-        /* we need to patch the image */
-        size_t size;
-        uint8_t *buf;
-        IsoStream *new = NULL;
-        IsoStream *original = t->bootimg->stream;
+    /* Patch the boot image info tables if indicated */
+    for (idx = 0; idx < t->catalog->num_bootimages; idx++) {
+        if (!(t->catalog->bootimages[idx]->isolinux_options & 0x01))
+    continue;
+        original = t->bootsrc[idx]->stream;
         size = (size_t) iso_stream_get_size(original);
+
+        /* >>> BOOT ts B00428 :
+               check whether size is not too large for buffering */;
+
         buf = calloc(1, size);
         if (buf == NULL) {
             return ISO_OUT_OF_MEM;
@@ -858,7 +1098,7 @@ int eltorito_writer_compute_data_blocks(IsoImageWriter *writer)
         }
 
         /* ok, patch the read buffer */
-        ret = patch_boot_image(buf, t, size);
+        ret = patch_boot_image(buf, t, size, idx);
         if (ret < 0) {
             return ret;
         }
@@ -869,7 +1109,7 @@ int eltorito_writer_compute_data_blocks(IsoImageWriter *writer)
         if (ret < 0) {
             return ret;
         }
-        t->bootimg->stream = new;
+        t->bootsrc[idx]->stream = new;
         iso_stream_unref(original);
     }
     return ISO_SUCCESS;
@@ -920,7 +1160,7 @@ int eltorito_writer_free_data(IsoImageWriter *writer)
 
 int eltorito_writer_create(Ecma119Image *target)
 {
-    int ret;
+    int ret, idx;
     IsoImageWriter *writer;
     IsoFile *bootimg;
     IsoFileSrc *src;
@@ -951,16 +1191,19 @@ int eltorito_writer_create(Ecma119Image *target)
             return ret;
         }
     }
-    bootimg = target->catalog->image->image;
-    ret = iso_file_src_create(target, bootimg, &src);
-    if (ret < 0) {
-        return ret;
-    }
-    target->bootimg = src;
 
-    /* if we have selected to patch the image, it needs to be copied always */
-    if (target->catalog->image->isolinux_options & 0x01) {
-        src->prev_img = 0;
+    for (idx = 0; idx < target->catalog->num_bootimages; idx++) {
+        bootimg = target->catalog->bootimages[idx]->image;
+        ret = iso_file_src_create(target, bootimg, &src);
+        if (ret < 0) {
+            return ret;
+        }
+        target->bootsrc[idx] = src;
+
+        /* For patching an image, it needs to be copied always */
+        if (target->catalog->bootimages[idx]->isolinux_options & 0x01) {
+            src->prev_img = 0;
+        }
     }
 
     /* we need the bootable volume descriptor */
