@@ -18,6 +18,11 @@
 #ifndef Xorriso_private_includeD
 #define Xorriso_private_includeD yes
 
+
+/* >>> transition to new boot image management */
+#define Xorriso_multi_booT 1
+
+
 /** The source code release timestamp */
 #include "xorriso_timestamp.h"
 #ifndef Xorriso_timestamP
@@ -259,16 +264,32 @@ struct XorrisO { /* the global context of xorriso */
  int stdio_sync;              /* stdio fsync interval: -1, 0, >=32 */
 
  int keep_boot_image;
- int patch_isolinux_image;
+ char boot_image_cat_path[SfileadrL];
+ int boot_count; /* number of already attached boot images */
+
  char boot_image_bin_path[SfileadrL];
  char boot_image_bin_form[16];
+ int boot_platform_id; 
+ int patch_isolinux_image; /* bit0= boot-info-table , bit1= not with EFI */
  int boot_image_emul; /* 0=no emulation
                         (1=emulation as hard disk)
                         (2=emulation as floppy)
                       */
- char boot_image_cat_path[SfileadrL];
  off_t boot_image_load_size;
- int boot_image_isohybrid; /* 0=off , 1=auto , 2=on , 3=force */
+ unsigned char boot_id_string[29];
+ unsigned char boot_selection_crit[21];
+
+ int boot_image_isohybrid; /* 0=off , deprecated: 1=auto , 2=on , 3=force */
+
+ int boot_efi_default; /* 0= no effect ,
+                          1= appy --efi-boot parameters when attaching to img */
+
+#ifndef Xorriso_multi_booT
+ /* <<< BOOT replace by attaching previous boot image to IsoImage by
+        -boot_image any next */
+ char boot_image_efi_path[SfileadrL];
+#endif /* ! Xorriso_multi_booT */
+
  char system_area_disk_path[SfileadrL];
  int system_area_options;             /* bit0= "GRUB protective msdos label"
                                                (a simple partition table)
