@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  0
 #define xorriso_libisoburn_req_minor  5
-#define xorriso_libisoburn_req_micro  4
+#define xorriso_libisoburn_req_micro  6
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -491,7 +491,17 @@ int Xorriso_set_change_pending(struct XorrisO *xorriso, int flag);
 /* @param flag bit0= enable SCSI command logging to stderr */
 int Xorriso_scsi_log(struct XorrisO *xorriso, int flag);
 
+/* flag bit0= do not increment boot_count and do not reset boot parameters
+        bit1= dispose attached boot images
+*/
+int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag);
 
+/*
+ bit0= do only report non-default settings
+ bit1= do only report to fp
+*/
+int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
+                              int flag);
 
 /* A pseudo file type for El-Torito bootsectors as in man 2 stat :
    For now take the highest possible value.
